#include <cstdlib>
#include <iostream>

using namespace std;

// Constants decreed by the Head-Monkey
#define PI_BY_2 102944
#define PI      205887
#define _2_PI   411775

class FPA {
public:
   long long value;             // Value of number

   FPA();                       // Default Constructor
   FPA(long long init);
   FPA mul(const FPA& arg);
   FPA div(const FPA& arg);
   // Overloaded; two arguments
   FPA mul(const FPA& x, const FPA& y);
   FPA div(const FPA& x, const FPA& y);
   FPA& operator =(const FPA& arg);
   FPA sin();
   FPA cos();
   double toDouble();
   
   // The more *advanced* skidding methods for FPA
   FPA abs();
   FPA sqrt();
   FPA asin();
   FPA atan();
   FPA atan2(const FPA& y);  
   FPA anglediff(const FPA& y);
};

FPA operator +(const FPA& x, const FPA& y);
FPA operator -(const FPA& x, const FPA& y);
FPA operator -(const FPA& arg);   // Unary minus

FPA::FPA() {
   value = 0;
}

FPA::FPA(long long init) {
   value = init;
}

FPA operator -(const FPA& arg) {   // Unary minus
   return FPA(-arg.value);
}

double FPA::toDouble()
{
  return (double)((*this).value) / 65536;
}

FPA FPA::mul(const FPA& arg) {
   return FPA((value * arg.value) >> 16);
}

FPA FPA::div(const FPA& arg) {
   return FPA((value << 16) / arg.value);
}

FPA FPA::mul(const FPA& x, const FPA& y) {
   return FPA((x.value * y.value) >> 16);
}

FPA mul(const FPA& x, const FPA& y) {
   return FPA((x.value * y.value) >> 16);
}

FPA FPA::div(const FPA& x, const FPA& y) {
   return FPA((x.value << 16) / y.value);
}

FPA div(const FPA& x, const FPA& y) {
   return FPA((x.value << 16) / y.value);
}

// Probably don't need this, but what the hell...
FPA& FPA::operator=(const FPA& arg) {
   if (this == &arg) return *this; // i.e. left side == right side
   else {
      value = arg.value;
      return *this;
   }
}

FPA operator+(const FPA& x, const FPA& y) {
   return FPA(x.value + y.value);
}

FPA operator-(const FPA&x, const FPA& y) {
   return FPA(x.value - y.value);
}

FPA FPA::sin() {
   if (value < 0) return -(FPA(-value)).sin();
   if (value > PI_BY_2) return (FPA(PI - value)).sin();
   FPA x2 = mul(*this, *this);
   FPA x3 = mul(*this, x2);
   FPA x5 = mul(x3, x2);
   FPA x7 = mul(x5, x2);
   return (value - (x3.value)/6 + (x5.value)/120 - (x7.value)/5040);
}

FPA FPA::cos() {
   value += PI_BY_2;
   if (value > PI) value -= _2_PI;
   return (*this).sin();
}

FPA FPA::abs() {
   if (value > 0) return FPA(value); else return FPA(-value);
}

FPA FPA::sqrt() {
   long u = value > 1 ? value : 1;
   long l = 0;
   while (u != l) {
      FPA g = div(FPA(u) + FPA(l), FPA(2));
      if ((mul(g, g).value) > value) u = g.value;
      else l = g.value + 1;
   }
   return FPA(u);
}

FPA FPA::asin() {
   FPA tmp;
   tmp = tmp - (div(tmp.sin() - *this, tmp.cos()));
   tmp = tmp - (div(tmp.sin() - *this, tmp.cos()));
   tmp = tmp - (div(tmp.sin() - *this, tmp.cos()));
   return tmp;   
}

FPA FPA::atan() {
   if (value < 0) return -(FPA(-value)).atan();
   if (value > 1) return (PI_BY_2 - (div(FPA(1), *this))).atan();
   return (div(*this, (FPA(1) + mul(*this, *this)).sqrt())).asin();
}

FPA makeFpa(long long v)
{ return FPA( v << 16 ); }

FPA accel(FPA& vo)
{ return vo + FPA(24); }

FPA brake(FPA& vo)
{ return vo - FPA(36); }

FPA friction(FPA& vo)
{ return vo - FPA(4) - mul( FPA(12), vo ) - mul( FPA(24), mul(vo,vo) ); }

int main()
{
  cout << ((FPA)div( FPA(64), FPA(0) + FPA(20000) )).value << '\n';
  FPA f = div( FPA(64), FPA(0) + FPA(20000) );
  cout << "Pi by 2: " << FPA(PI_BY_2).toDouble() << '\n';
  cout << "f: " << f.toDouble() << '\n';
  cout << "f (degrees): " << f.toDouble()*180/3.14 << " [floating point c]\n";
  cout << "f (degrees): " << f.mul(180 << 16).div(PI).toDouble() << " [fixed point]\n";
  FPA v = FPA(0), vo = FPA(0);
//  int a = 0;
  do
  {
    vo = v;
//    a++;
    v = accel(vo);
  } while ( v.value != vo.value );
  cout << "terminal velocity: " << v.value << " [in FPA],  " << v.toDouble() << " [in real]\n";
//  cout << "took " << a <<" steps.\n";
}
