import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

public class MapViewer extends Frame implements WindowListener, MouseListener
{
  public static final int ROAD = 0x000000;
  public static final int R = 0x880000;
  public static final int G = 0x008800;
  public static final int B = 0x000088;
  public static final int W = 0x888888;
  public static final int START = 0x00ffff;
  public static final int FINISH = 0xaaff00;
  public static final int PATH = 0xff00ff;
  public static final int PLANP = 0xffaa00;
  public static final int PLANL = 0xffaa00;
  public static final int MAPWIDTH = 1024;
  public static final int MAPHEIGHT = 768;

  public static final int ERROR = 0xff0000;
  BufferedImage image, theMap;
  Point startPt;
  String fileName;

  public static BufferedImage getImageFromFile( String file )
  { return getImageFromFile( file, null, null ); }
  
  public static BufferedImage getImageFromFile( String file, Point startPt, ArrayList endPts )
  {
    try
    {
      BufferedReader reader = new BufferedReader(new FileReader(file));
      if ( Integer.parseInt(reader.readLine()) != MAPWIDTH)
        System.out.println ("file width != map width");
      if ( MAPHEIGHT != Integer.parseInt(reader.readLine()))
        System.out.println ("file height != map height");
      BufferedImage image = new BufferedImage(MAPWIDTH, MAPHEIGHT, BufferedImage.TYPE_INT_RGB);
      for(int y=0; y<MAPHEIGHT; y++)
      {
        String line = reader.readLine();
        for(int x=0; x<MAPWIDTH; x++)
        {
          switch(line.charAt(x))
          {
            case '.':                            
                image.setRGB(x, y, ROAD); //black
                break;
            case '*':
                image.setRGB(x, y, START); //yellow
                if (startPt != null)
                  startPt.setLocation(x,y);
                break;
            case '!':
                image.setRGB(x, y, FINISH); //yellow
                if (endPts != null)
                  endPts.add(new Point(x,y));
                break;
            case 'g':
                image.setRGB(x, y, G); //green
                break;
            case 'r':
                image.setRGB(x, y, R); //red
                break;
            case 'w':
                image.setRGB(x, y, W); //white
                break;
            case 'b':
                image.setRGB(x, y, B); //blue
                break;
            default:
                image.setRGB(x, y, ERROR);  //orange
                break;
          }
        }
      }
      return image;
    }
    catch(Exception e)
    { 
      System.out.println("Error parsing map");
      e.printStackTrace();
    }
    return null;
  }
  
  public void redrawImage()
  {
    Insets insets = getInsets();
    getGraphics().drawImage(image,insets.left,insets.top,this);
  }

  public void putPixel(int x, int y, int col)
  { this.image.setRGB(x,y,col); }

  public void putPixelRedraw(int x, int y, int col)
  { putPixel(x,y,col); redrawImage(); }
  
  MapViewer(String file)
  {
    super("Race Track - "+file);
    fileName = file;
    startPt = new Point();
    image = getImageFromFile(file,startPt,null);
    theMap = getImageFromFile(file);
    if (image == theMap) System.out.println("ERROR");
    setVisible(true);
    Insets insets = getInsets();
    setSize(MAPWIDTH+insets.left+insets.right, MAPHEIGHT+insets.top+insets.bottom);
    Graphics graphics = getGraphics();
    graphics.drawImage(image,insets.left,insets.top,this);
    addWindowListener(this);
    addMouseListener(this);
  }

  public void plotPath(long x, long y)
  { putPixel((int)x,(int)y,PATH); }
  
  public void windowClosing(WindowEvent e)
  { dispose(); }
  
  // I don't care about these events, but I have to implement them
  // because I'm a WindowListener.
  public void windowOpened(WindowEvent e){}
  public void windowClosed(WindowEvent e){}
  public void windowIconified(WindowEvent e){}
  public void windowDeiconified(WindowEvent e){}
  public void windowActivated(WindowEvent e) {}
  public void windowDeactivated(WindowEvent e){}

  public void mouseClicked(MouseEvent e) {}
  public void mousePressed(MouseEvent e) {}
  public void mouseReleased(MouseEvent e) {}
  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}
  
}

