import java.awt.event.*;
import java.util.ArrayList;
import java.awt.*;
import java.awt.image.*;
import java.io.*;

public class Mapper extends MapViewer implements MouseMotionListener, KeyListener
{
  public static final int PSIZE = 3;
  public ArrayList points;
  public ArrayList handles;
  public Point curPt;
  public boolean mdown, moved, isHandle;
  public int handleNum;

  public Mapper(String map)
  { 
    super(map);
    if (image == null) System.out.println("Bad image");
    points = new ArrayList();
    handles = new ArrayList();
    addMouseMotionListener(this);
    addKeyListener(this);
    points.add(startPt);
    handles.add( new Point((int)startPt.getX()+10,(int)startPt.getY()));
    handles.add( new Point((int)startPt.getX()-10,(int)startPt.getY()));
  }
    
  public static void main(String[] args) throws Exception
  {
    if (args.length < 1)
    {
      System.out.println("Usage 'java Mapper <map-file>'");
      System.exit(1);
    }
    Mapper m = new Mapper(args[0]);
    m.redrawImage();
    m.redrawImage();
    Thread.sleep(3);
    m.redrawImage();
  }

  public Point nearest(int x, int y)
  {
    int size=15;
    double dist=size*size*50;
    Point p = new Point(x,y);
    Point q = new Point(x,y);
    for(int i=x-size;i<=x+size;i++)
      for(int j=y-size;j<=y+size;j++)
      {
        if (((theMap.getRGB(i,j)&0xFFFFFF) == ROAD) && (q.distance(i,j) < dist))
        {
          p.setLocation(i,j);
          dist=q.distance(i,j);
        }
      }
    return p;
  }

  public void mousePressed(MouseEvent e)
  {
    curPt = null;
    moved = false; 
    int colour, handleSize;
    int x, y;
    x = e.getX()-getInsets().left;
    y = e.getY()-getInsets().top;
    handleSize = 25;
    
    if (e.getButton() == e.BUTTON3)
    {
      Point p = nearest(x, y);
      colour = image.getRGB((int)p.getX(),(int)p.getY()) & 0xFFFFFF;
      if ( colour == ROAD )
      {
        Point h1 = new Point((int)p.getX()+handleSize, (int)p.getY());
        Point h2 = new Point((int)p.getX()-handleSize, (int)p.getY());
        
        points.add(p);
        handles.add(h1);
        handles.add(h2);
        //redrawPoints(p);
        redrawPoints();
      }
    }
    else if (e.getButton() == e.BUTTON1)
    {
      mdown = true;
      curPt = null;
      isHandle = false;
      handleNum = -1;
      for (int i=0; i<points.size(); i++)
        if ( ((Point)points.get(i)).distance(x,y) <= PSIZE )
        {
            handleNum = i;
            curPt = (Point)points.get(i);
            break;
        }
      if (curPt==null)
        for (int i=0; i<handles.size(); i++)
          if ( ((Point)handles.get(i)).distance(x,y) <= PSIZE )
          {
            isHandle = true;
            handleNum = i;
            curPt = (Point)handles.get(i);
            break;
          }
    }
  }

  public void redrawPoints()
  {
    int i;

    image.setData(theMap.getData());
    image.getGraphics().setColor(new Color(PLANL));
    for (i=0; i<handles.size()/2; i++)
    {
      Point h1 = (Point) handles.get(i*2);
      Point h2 = (Point) handles.get(i*2+1);
      image.getGraphics().drawLine((int)h1.getX(),(int)h1.getY(),(int)h2.getX(),(int)h2.getY());
    }
    image.getGraphics().setColor(new Color(PLANP));
    for (i=0; i<points.size(); i++)
      drawPoint( (Point)points.get(i) );
    for (i=0; i<handles.size(); i++)
      drawPoint( (Point)handles.get(i) );

    //draw curve
    for (i=0; i<points.size()-1; i++)
    {
      Point a, b, c, d;
      a = (Point) points.get(i);
      b = (Point) handles.get(i*2);
      c = (Point) handles.get(i*2+3);
      d = (Point) points.get(i+1);
      
      Point p1, p2;
      double x, y, t;
      p1 = new Point(d);
      p2 = new Point(d);
      for (int j=1; j<=100; j++)
      {
        t = j/100.0;
        
        x = a.getX()*Math.pow(t,3) + 
          3.0*b.getX()*Math.pow(t,2)*(1-t) + 
          3.0*c.getX()*Math.pow(1-t,2)*t +
            d.getX()*Math.pow(1-t,3);
        y = a.getY()*Math.pow(t,3) + 
          3.0*b.getY()*Math.pow(t,2)*(1-t) + 
          3.0*c.getY()*Math.pow(1-t,2)*t +
            d.getY()*Math.pow(1-t,3);
        
        p2.setLocation(x, y);
        image.getGraphics().drawLine((int)p1.getX(), (int)p1.getY(), (int)p2.getX(), (int)p2.getY());
        p1 = new Point(p2);
      }
    }
    
    redrawImage();
  }

  public void drawPoint( Point p )
  {
    image.getGraphics().fillOval((int)p.getX()-PSIZE,(int)p.getY()-PSIZE,2*PSIZE,2*PSIZE);
  }

  public void mouseReleased(MouseEvent e)
  {
    mdown = false;
    if (moved)
      redrawPoints();
    moved = false;
  }

  
  public void mouseMoved(MouseEvent e)
  {
  }

  public void mouseDragged(MouseEvent e)
  {
    int x, y;
    int pointNum, other;
    x = e.getX()-getInsets().left;
    y = e.getY()-getInsets().top;
    
    if (curPt == null) return;
    
    if (!isHandle)
    {
      Point p = nearest(x,y);
      Point h1 = (Point) handles.get(handleNum*2);
      Point h2 = (Point) handles.get(handleNum*2+1);
      
      if ((theMap.getRGB((int)p.getX(),(int)p.getY())&0xFFFFFF) == ROAD)
      {
        h1.setLocation((int)h1.getX()-curPt.getX()+p.getX(), (int)h1.getY()-curPt.getY()+p.getY());
        h2.setLocation((int)h2.getX()-curPt.getX()+p.getX(), (int)h2.getY()-curPt.getY()+p.getY());
        curPt.setLocation(p);
      }
    }
    else
    {
      Point p = new Point(x,y);
      pointNum = handleNum/2;
      other = pointNum*2 + (handleNum+1)%2;

      Point pnt = (Point) points.get(pointNum);
      Point h2 = (Point) handles.get(other);
      double len1, len2, newLen;
      len1 = pnt.distance(p);
      len2 = pnt.distance(h2);
      if (len1!=0)
        newLen = -len2/len1;
      else
        newLen = -5;
      
      Point dir = new Point((int)(p.getX()-pnt.getX()), (int)(p.getY()-pnt.getY()));
      h2.setLocation(dir.getX()*newLen+pnt.getX(), dir.getY()*newLen+pnt.getY());
      
      curPt.setLocation(p);
    }
    //redrawPoints(curPt);
    redrawPoints();
    moved=true;
  }

  public void keyPressed(KeyEvent e)
  {}

  public void keyReleased(KeyEvent e)
  {
    try
    {
      if (e.getKeyChar() == 's')
      {
        ObjectOutputStream oos = new ObjectOutputStream( new FileOutputStream( "pts-"+fileName ) );
        oos.writeObject( points );
        oos = new ObjectOutputStream( new FileOutputStream( "hndls-"+fileName ) );
        oos.writeObject( handles );
      }
      if (e.getKeyChar() == 'l')
      {
        ObjectInputStream oos = new ObjectInputStream( new FileInputStream( "pts-"+fileName ) );
        points = (ArrayList)oos.readObject();
        oos = new ObjectInputStream( new FileInputStream( "hndls-"+fileName ) );
        handles = (ArrayList)oos.readObject( );
        redrawPoints();
      }
    }
    catch (Exception oe)
    { System.out.println("Object is juiced: "+oe); }
  }

  public void keyTyped(KeyEvent e)
  {}
}
