import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

public class TrackViewerWindow extends MapViewer implements Runnable, MouseListener
{
  public static final int REFRESH_TIME = 50;
  public static final int TIME_STEP_NANO = 150;

  public ArrayList[][] pixels = new ArrayList[MAPWIDTH][];
  
  public TrackViewerWindow(String map)
  { 
    super(map); 
    addMouseListener(this);
  }

  public Point nearest(int x, int y)
  {
    int size=15;
    double dist=1e20;
    Point p = new Point(x,y);
    Point q = new Point(x,y);
    for(int i=x-size;i<=x+size;i++)
      for(int j=y-size;j<=y+size;j++)
      {
        if (((image.getRGB(i,j)&0xFFFFFF) == PATH) && (q.distance(i,j) < dist))
        {
          p.setLocation(i,j);
          dist=q.distance(i,j);
        }
      }
    return p;
  }

  public void showMovement(String dumpFile)
  {
    for (int i=0; i<MAPWIDTH; i++)
      pixels[i] = new ArrayList[MAPHEIGHT];
    
    try
    {
      BufferedReader reader = new BufferedReader(new FileReader(dumpFile));
      reader.readLine();
      StringTokenizer st;
      int x, y;
      String line;
      while((line=reader.readLine())!=null)
      {
        st = new StringTokenizer(line, " :");
        st.nextToken();
        int fpx,fpy;
        x = (fpx=Integer.parseInt(st.nextToken()))/65536;
        y = (fpy=Integer.parseInt(st.nextToken()))/65536;
        if ( x > 0 && x < 1024 && y > 0 && y < 768 )
        {
          plotPath(x,y);
          if (pixels[x][y] == null)
            pixels[x][y] = new ArrayList();
          pixels[x][y].add(new Point(fpx,fpy));
          Thread.sleep(0,TIME_STEP_NANO);
        }
        else
        {
          System.out.println("Off path :"+x+","+y);
          break;
        }
      }
    }
    catch(Exception e)
    {
      System.out.println("Error in showMovement");
      e.printStackTrace();
    }
  }

  public void showMovement( ArrayList pts, int col )
  {
    for (int i=0; i<pts.size(); i++)
    {
      Point p = (Point)pts.get(i);
      putPixel((int)p.getX(),(int)p.getY(),col);
    }
  }
  
  public void run()
  {
    try
    {
      while (true)
      { 
        redrawImage(); 
        Thread.sleep(REFRESH_TIME); 
      }
    }
    catch (Exception e) 
    { 
      System.out.println("Run terminated: "+e); 
      e.printStackTrace(); 
    }
  }

  public void mousePressed(MouseEvent e)
  {
  }
  public void mouseClicked(MouseEvent e)
  {
    int x = (int)e.getX()-getInsets().left, y = (int)e.getY()-getInsets().top;
    if (e.getButton() != e.BUTTON1) return;
    System.out.println("You clicked on ("+x+","+y+")");
    Point z = nearest(x,y);
    System.out.println("You may be more interested in "+z+"\n\t"+pixels[x][y]);
    new Pixie( pixels[(int)z.getX()][(int)z.getY()] );
  }
  public void mouseReleased(MouseEvent e)
  {
    /*int x = (int)e.getX()-getInsets().left, y = (int)e.getY()-getInsets().top;
    if (e.getButton() != e.BUTTON1) return;
    System.out.println("You clicked on ("+x+","+y+")");
    Point z = nearest(x,y);
    System.out.println("You may be more interested in "+z+"\n\t"+pixels[x][y]);
    new Pixie( pixels[(int)z.getX()][(int)z.getY()] );*/
  }
  public void mouseEntered(MouseEvent e)
  {
  }
  public void mouseExited(MouseEvent e)
  {
  }
}

